/**
  * NOTE TO DEVELOPER:
  * This is a generic snap applet component. It contains the 
  * minimum implementation required for a third-party component
  * to be snapable. The Snapplet implements the Snapable
  * interface. The developer may use this class in the following
  * ways:
  *
  * 1. Extend Snapplet and fill in the required implementation details.
  *
  * 2. Implement the Snapable interface and reuse the code in Snapplet.
  */

package snap;

import snap.*;

//Standard Java classes
import java.util.*;
import javax.swing.JApplet;
import java.sql.ResultSet;
import javax.swing.Icon;

/** An abstract applet that implements the Snapable interface.
  * The developer must fill in some of the functionality that
  * is component-specific.
  *
  * @author Nathan Conklin - nathan@conklinfamily.net
  * @author Chris North - north@cs.vt.edu
  */
public abstract class Snapplet extends JApplet
implements Snapable
{
	/** Container to keep track of the registered listeners. */
	protected Vector listeners = new Vector();

	/** Adds a listener for snap events. */
	public void addSnapEventListener(SnapEventListener sel) {
		listeners.add(sel);
	}

	/** Removes a listener for snap events. */
	public void removeSnapEventListener(SnapEventListener sel) {
		listeners.remove(sel);
	}

	/** Fires a SnapEvent to all registered listeners. */
	public void fireSnapEvent(SnapEvent snapEvent)
	{
	   	Enumeration e = listeners.elements();
		while(e.hasMoreElements())
		{    		
			SnapEventListener sel = (SnapEventListener) e.nextElement();
			sel.snapEventOccured(snapEvent);
		}
	}

	/** Returns null, setting the Icon to a default Icon.  This method should
	  * be overloaded in extending subclasses to return the Icon for the new
	  * visualization. */
	public Icon getIcon()
	{
		return (Icon) null;
	}

	/** Method called when Snap needs to load data into the visualization. */
	public abstract void load(ResultSet rs, String primaryKeyColumnName);

	/** Returns an Enumeration of actions (as Strings) that the component can
	  * perform and will fire as SnapEvents. */
	public abstract Enumeration getSupportedActions();

	/** Executes the appropriate action for the SnapEvent. */
	public abstract void performSnapEvent(SnapEvent e);


}