package snap.coordination;

import snap.*;
import snap.database.*;
import snap.adapter.*;
import snap.loader.*;
import snap.graph.*;

import java.sql.*;
import java.util.*;
import java.beans.*;

/**
  * This is the controlling class to manage the connections and
  * visualization layout.
  *
  * @author Varun Saini - vsaini@vt.edu 
  * @author Chris North - north@cs.vt.edu
  * @author Nathan Conklin - nathan@conklinfamily.net
  * @author Umer Farooq - ufarooq@vt.edu
  */
public class CoordinationManager
implements SnapEventListener, GraphChangeEventListener
{

private KeyConverter kc; // Handle to key converter
private Graph graph; // Graph structure
private Snap snap;  // Snap application

/**
 * Constructor.
 * Builds the CoordinationManager.
 */
public CoordinationManager(Snap snap)
{
	this.snap = snap;
	this.graph = snap.getGraph();
	kc = new KeyConverter(snap.getDatabaseManager());
}

/**
 * Method for the graph changed event.
 * Called when the graph is changed, e.g.
 * addition/deletion of nodes.
 */
public void graphChanged(GraphChangeEvent e)
{
	if(e.getEventType() == GraphChangeEvent.NODE_ADDED)
	{
		TechnologyAdapter adapter = e.getNode().getTechnologyAdapter();
		adapter.addSnapEventListener(this);
	}
	else if(e.getEventType() == GraphChangeEvent.NODE_REMOVED)
	{
		TechnologyAdapter adapter = e.getNode().getTechnologyAdapter();
		adapter.removeSnapEventListener(this);
	}
}

/** 
 * Finds the SnapEvent to be fired on the nextNode.
 */
private SnapEvent getNextEvent(SnapEvent sourceEvent, CoordinationNode nextNode,
							   String eventToFire)
{
	// Getting the source node from the Snap event
	TechnologyAdapter sourceAdapter = 	
		(TechnologyAdapter)sourceEvent.getSource();
	CoordinationNode sourceNode =
		(CoordinationNode)sourceAdapter.getNode();

	// Src query
	Query srcQuery = sourceNode.getQuery();
	// Dest query
	Query destQuery = nextNode.getQuery();		

	// NOTE: This can be used to drive Snap without key conversion when 
	// uncommented, and commenting the next line.
	// Used to pass around the global event, no key conversion
	// SnapEvent se = sourceEvent;

	// Calling key converter and getting new Snap event
	SnapEvent se = kc.convertKeys(srcQuery, destQuery, sourceEvent);

	// Create the new event to be fired because Snap event attained in above
	// call to key converter does not have the technology adapter and the 
	// type of event set
	SnapEvent destEvent = new SnapEvent(nextNode.getTechnologyAdapter(),
										eventToFire, se.getKeys());

	return destEvent;
}

/** 
 * This method implements the marking within the mark and sweep algorithm.
 */
private void markNodes(SnapEvent sourceEvent, Vector connectedNodes)
{
	// Keeps track of nodes marked in this iteration
	Vector markedNodes = new Vector();

	// Two while loops needed for breadth first traversal
	Enumeration e = connectedNodes.elements();
	while(e.hasMoreElements())
	{
		// Getting the node action pair
		NodeActionPair nodeActionPair = (NodeActionPair) e.nextElement();	
		CoordinationNode nextNode = (CoordinationNode) nodeActionPair.getNode();

		// If node not marked, then mark it
		if (!nextNode.getMark())
		{
			nextNode.setMark(true);

			// Adds nextNode as a markedNode
			markedNodes.add(nextNode);

			// Sets the SnapEvent in the Node
			SnapEvent se = getNextEvent(
				sourceEvent, nextNode, nodeActionPair.getAction());
			nextNode.setSnapEvent(se);

			// This fires the event right after marking the node
			(nextNode.getTechnologyAdapter()).performSnapEvent(
				nextNode.getSnapEvent());
		}
	}

	// This loop is the event propogation and
	// maintains the breadth-first order
	e = connectedNodes.elements();
	while(e.hasMoreElements())
	{
		NodeActionPair nodeActionPair = (NodeActionPair)e.nextElement();
		CoordinationNode nextNode = 
			(CoordinationNode) nodeActionPair.getNode();

		if (markedNodes.contains(nextNode)) 
		{
			SnapEvent eventToPropagate = nextNode.getSnapEvent();
			markNodes(eventToPropagate, nextNode.getConnections(
				nodeActionPair.getAction()));
		}
	}
}

/**
 * This method is called when a Snap event occurs.
 */
public void snapEventOccured(SnapEvent userEvent)
{
	SnapEvent componentEvent = cleanUpIncomingSnapEvent(userEvent);
	try
	{
		// Get the read lock
		// Critical section starts
		graph.getMonitor().acquireRead();

		TechnologyAdapter sourceAdapter =
			(TechnologyAdapter) componentEvent.getSource();

		// Get the initial node which was the source of the event being handled
		CoordinationNode sourceNode =
			(CoordinationNode) sourceAdapter.getNode();

		// Mark this node
		sourceNode.setMark(true); 
		
		// Get the action which caused the event
		String action = componentEvent.getEventType();

		// Mark the connecting nodes and fire the events
		Vector connectedNodes = sourceNode.getConnections(action);

		markNodes(componentEvent, connectedNodes);

		// Removes the marks and cleans the Coordination graph
		unMarkNodes();
	}
	catch(Exception ex)
	{
		ex.printStackTrace();
		
	}
	finally
	{
		// Release read lock
		// Critical section ends
		graph.getMonitor().releaseRead();
	}
}

/** Clears the marked nodes within the CoordinationGraph */
private void unMarkNodes()
{
	Enumeration e = graph.nodes();
	while (e.hasMoreElements())
	{
		CoordinationNode node = (CoordinationNode)e.nextElement();
		node.setSnapEvent(null);
		node.setMark(false);				
	}
}

/** Used to make sure that events fired at Snap are "good events" */
private SnapEvent cleanUpIncomingSnapEvent(SnapEvent se)
{
	Vector keys = se.getKeys();
	if (keys == null)
		keys = new Vector();
	String eventType = se.getEventType();
	if (eventType == null)
		eventType = "";
	return new SnapEvent(se.getSource(), eventType, keys);
}

}
