
package snap.loader;

import snap.adapter.*;
import snap.graph.*;
import snap.*;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.Enumeration;

/** An applet that runs on the same page as a Java Applet visualization
  * component.  The AppletLoader with find the applet on the page and will 
  * register that component with Snap.
  *
  * @author Nathan Conklin - nathan@conklinfamily.net
  * @author Chris North - north@cs.vt.edu
  */
public class AppletLoader extends JApplet
{

protected boolean inAnApplet = true;  // distinguish from application
protected JTextArea textArea;

private String componentType = null;
private Object component = null;
private String componentName = null;

private Snap snap;
private JavaTechnologyAdapter adapter; 


/** Finds and loads a Snapable applet */
public void start()
{
    findApplet();
	if(component != null)
		loadComponent(componentType, component);
}

/** Unloads the Snapable applet */
public void stop()
{
	if(componentName != null)
	{
		try
		{
			UIGraph graph = (UIGraph) snap.getGraph();
			graph.removeComponent(componentName);
		}
		catch(Exception e)
		{
			String error = 
				"Error talking to Snap or trying to unload: " +
				componentName + " - " + e.toString();
			System.err.println(error);
		}
	}
}

/** Attempts to add the component to Snap.  Returns the name of the
  * component as defined by Snap. */
private void loadComponent(String componentType, Object component)
{
	Snapable snapComponent;
	String name;

	try
	{
		snap = (Snap) StaticMemoryList.getManager();
		snapComponent = (Snapable) component;
		adapter = new JavaTechnologyAdapter(snap, snapComponent);
		UIGraph graph = (UIGraph) snap.getGraph();
		componentName = graph.addComponent(componentType, adapter);
	}
	catch(Exception e)
	{
		String error = 
			"Error talking to Snap or trying to load a component " +
			"that is not Snapable: " + e.toString();
		System.err.println(error);
		e.printStackTrace();
	}
}

/** Gets the applet handle from the webpage.  Sets component to the applet
  * handle and sets the componentType */
private void findApplet()
{
    AppletContext ac;
    Applet a;
    int i = 0;
    int maxTries = 10;

    // The applet's name will be passed as a parament in the HTML
    String nameTag = getParameter("Component");
    if(nameTag == null)
    {
        String msg = "No applet name passed as a Component\n" +
		"Must include an applet PARAM tag naming another Component" + 
		" on page\nExample:\n\n\n" +
		"<APPLET CODE=Table.class NAME=\"Table\"></APPLET>\n" +
		"<P>\n"+
		"<APPLET CODE=AppletLoader.class NAME=\"AppletLoader\">\n"+
		"<PARAM NAME=\"Component\" VALUE=\"Table\">\n" +
		"</APPLET>\n";
        setText(msg);
	return;
    }

    try
    {
        ac = getAppletContext();  // Retrieves environment information
		a = ac.getApplet(nameTag); // Attempts to get the applet
		while(a == null) // May need to try again if applet has not loaded
		{
			i++;
			if(i == maxTries)
				break;
			Thread.sleep(1000);  // 1000 milliseconds
			a = ac.getApplet(nameTag);
		}
		if(i == maxTries)  // Applet never found
		{
			String error = "Unable to connect to applet with NAME = \"" +
			nameTag + "\"\n" +
			"The applet may have never initialized or the NAME " +
			"parameter may not be set.";
			setText(error);
		}
		else  // Applet was found and should be registered
		{
			componentType = a.getAppletInfo();
			if(componentType == null)
				componentType = a.getClass().getName();
			setText(nameTag + ": "+ componentType + " - found.");
			component = a;
		}

    }
    catch(Exception e)
    {
        setText("AppletLoader.findApplet() failed: " +
	    e.toString());
    }
}

/** Sets the text to be displayed in the AppletLoader */
public void setText(String text)
{
    textArea.setText(text);
}

/** Returns the text displayed for the AppletLoader */
public String getText()
{
    return textArea.getText();
}

/** Returns the AppletInfo for the AppletLoader */
public String getAppletInfo()
{
    return "AppletLoader by Nathan Conklin";
}

/* **************************************************************** */
/* Code to set up AppletLoader as an applet or application          */
/* **************************************************************** */

/** JApplet's init() method. This should not be explicitly called. **/
public void init()
{
    setContentPane(makeContentPane());
}

/** Builds the AppletLoader content pane with a TextArea that may be
used for basic output. */
protected Container makeContentPane()
{
    JPanel pane = new JPanel();
    pane.setLayout(new BorderLayout());
    textArea =
		new JTextArea("Snap's AppletLoader - Must be run as an applet");
    JScrollPane scrollPane = new JScrollPane(textArea);
    scrollPane.add(pane);
    scrollPane.setBorder(new EmptyBorder(0, 0, 0, 0));
    return scrollPane;
}

/** Constructor and applet initialization */
public AppletLoader()
{
    inAnApplet = true;
}

/** Constructor and applet initialization.  inAnApplet is true if
AppletLoader has been started as an applet */
public AppletLoader(boolean inAnApplet)
{
    this.inAnApplet = inAnApplet;
    if(inAnApplet)
    {
        getRootPane().putClientProperty(
            "defeatSystemEventQueueCheck", Boolean.TRUE);
    }
}

/** Starts up the applet if it is not in a browser */
public static void main(String args[])
{
    JFrame frame = new JFrame("AppletLoader");

    frame.addWindowListener(new WindowAdapter()
    {
        public void windowClosing(WindowEvent e)
	{
            System.exit(0);
	}
    }
    );

    AppletLoader applet = new AppletLoader(false);
    frame.setContentPane(applet.makeContentPane());
    frame.pack();
    frame.setVisible(true);
}

};
