package snap.ui.diagram;

import snap.graph.*;
import snap.database.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
  * CoordinationSettings is the component that obtains coordination
  * information from the user.  This configuration window determines what
  * actions the user would like to associate with each visualization 
  * involved for that particular link.  It prefills the visualization
  * type and allows the  user to type a coordination name to be associated
  * with the Coordination object.
  *
  * @author Nathan Conklin - nathan@conklinfamily.net
  * @author Matt Clement - macleme2@vt.edu
  * @author Chris North - north@cs.vt.edu
  */
public class CoordinationSettings extends JFrame
implements ActionListener
{

private DiagramPanel parent;
private DatabaseSchema schema;

private JPanel framePanel;	// Panel loaded by the frame

private JTextField coordinationName;	// Name of the coordination

private JComboBox actionFrom;	// First action
private JComboBox actionTo;		// Second action

private JTextField typeFrom;	// First visualization type
private JTextField typeTo;		// Second visualization type

private JTextField tableFrom;	// Table wrapped by first visualization
private JTextField tableTo;		// Table wrapped by second visualization

private JLabel relationshipFrom;	// 1, M, or empty
private JLabel relationshipTo;		// 1, M, or empty

private JPanel buttonPanel;

private boolean addingCoordination;  // true if adding a new coordination

private JButton saveButton;
private JButton cancelButton;    

private Link link;
private Coordination coordination;

private final static int WINDOW_WIDTH = 570;
private final static int WINDOW_HEIGHT = 230;

/**
 * This constructor takes the Link to add a Coordination to
 * 
 * @param link					the Link object to begin editing
 */          
public CoordinationSettings(Link link, DiagramPanel parent,
	 DatabaseSchema schema)
{
	this(link, null, parent, schema);
}

/**
 * This constructor takes the Link and Coordination objects to edit
 * 
 * @param link					the Link object to begin editing
 * @param coordination			the coordination to edit
 */          
public CoordinationSettings(Link link, Coordination coordination,
	 DiagramPanel parent, DatabaseSchema schema)
{
	this.link = link;
	this.coordination = coordination;
	this.parent = parent;  // Used to remove a Link when cancelled
	this.schema = schema;

	addingCoordination = false;

	if (this.coordination == null)
	{	// Add a new coordination to the link
		this.coordination = new Coordination(link);
		addingCoordination = true;
	}

	framePanel = new JPanel();
	framePanel.setLayout(new BoxLayout(framePanel, BoxLayout.Y_AXIS));
	
	String title = "Coordination Properties";
	this.setTitle(title);

	this.createRootPane();
	this.frameInit();
	setupWindow();
}

/**
 * Retrieves the name of the first Visualization object in the
 * coordination.
 */    
protected String getFirstVisualizationName()
{
	return link.getStartVisualization().getUINode().getName();
}
    
/**
 * Retrieves the name of the action associated with the first
 * Visualization
 */    
protected String getFirstAction()
{
	return actionFrom.getSelectedItem().toString();
}
    
/**
 * Retrieves the name of the second Visualization object in the
 * coordination.
 */   
protected String getSecondVisualizationName()
{
	return link.getEndVisualization().getUINode().getName();
}
    
/**
 * Retrieves the name of the action associated with the second
 * Visualization
 */    
protected String getSecondAction()
{
	 return actionTo.getSelectedItem().toString();
}    

/**
 * Sets up the interaction window with its default constituant objects and
 * initializes them to their default state.  The window is made up of
 * several panels each containing related objects.
 */    
protected void setupWindow()
{
	// Setup Directions Panel
	JPanel textPanel = new JPanel(new BorderLayout());
		JTextArea textArea = new JTextArea(
			"Select the actions that will be coordinated between " +
			"the two visualizations:");
		textArea.setLineWrap(true);
		textArea.setEditable(false);
		textArea.setBackground(textPanel.getBackground());
		textArea.setBorder(BorderFactory.createEmptyBorder(5,5,10,5));
	textPanel.add(textArea);

	// Setup Coordination Name Panel
	JPanel coordinationNamePanel = new JPanel(new BorderLayout());
		JLabel nameLabel = new JLabel("Name:");
		coordinationName = new JTextField();
		coordinationName.setText(coordination.getName());
		coordinationName.setEditable(false);
		nameLabel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	coordinationNamePanel.add(nameLabel, BorderLayout.WEST);
	coordinationNamePanel.add(coordinationName, BorderLayout.CENTER);


	// Setup Action Panel
		// Setup First Visualization Panel
		JPanel visualizationAPanel = new JPanel(new BorderLayout());
			visualizationAPanel.setBorder(
				BorderFactory.createTitledBorder(getFirstVisualizationName()));
			JLabel typeALabel = new JLabel("Type:");
			JLabel tableALabel = new JLabel("Data Table:");
			JLabel actionALabel = new JLabel("Action:");

			JPanel visualizationALabelGrid = new JPanel(new GridLayout(3, 1));
			visualizationALabelGrid.add(typeALabel);
			visualizationALabelGrid.add(tableALabel);
			visualizationALabelGrid.add(actionALabel);
			visualizationALabelGrid.setBorder(
				BorderFactory.createEmptyBorder(0,5,0,5));

			typeFrom = new JTextField();
			typeFrom.setEditable(false);
			tableFrom = new JTextField();
			tableFrom.setEditable(false);
			actionFrom = new JComboBox();

			JPanel visualizationASettingsGrid = new JPanel(
				new GridLayout(3, 1));
			visualizationASettingsGrid.add(typeFrom);
			visualizationASettingsGrid.add(tableFrom);
			visualizationASettingsGrid.add(actionFrom);

		visualizationAPanel.add(
			visualizationALabelGrid, BorderLayout.WEST);
		visualizationAPanel.add(
			visualizationASettingsGrid,	BorderLayout.CENTER);
		visualizationAPanel.setPreferredSize(new Dimension(250, 100));


		// Setup Second Visualization Panel
		JPanel visualizationBPanel = new JPanel(new BorderLayout());
			visualizationBPanel.setBorder(
				BorderFactory.createTitledBorder(getSecondVisualizationName()));
			JLabel typeBLabel = new JLabel("Type:");
			JLabel tableBLabel = new JLabel("Data Table:");
			JLabel actionBLabel = new JLabel("Action:");

			JPanel visualizationBLabelGrid = new JPanel(new GridLayout(3, 1));
			visualizationBLabelGrid.add(typeBLabel);
			visualizationBLabelGrid.add(tableBLabel);
			visualizationBLabelGrid.add(actionBLabel);
			visualizationBLabelGrid.setBorder(
				BorderFactory.createEmptyBorder(0,5,0,5));

			typeTo = new JTextField();
			typeTo.setEditable(false);
			tableTo = new JTextField();
			tableTo.setEditable(false);
			actionTo = new JComboBox();

			JPanel visualizationBSettingsGrid = new JPanel(
				new GridLayout(3, 1));
			visualizationBSettingsGrid.add(typeTo);
			visualizationBSettingsGrid.add(tableTo);
			visualizationBSettingsGrid.add(actionTo);

		visualizationBPanel.add(
			visualizationBLabelGrid, BorderLayout.WEST);
		visualizationBPanel.add(
			visualizationBSettingsGrid,	BorderLayout.CENTER);
		visualizationBPanel.setPreferredSize(new Dimension(250, 100));

		// Setup Relationship Panel (Draws the line between two components)
		JPanel relationshipPanel = new JPanel(new BorderLayout());
			relationshipFrom = new JLabel("");
			relationshipTo = new JLabel("");
			relationshipFrom.setVerticalAlignment(SwingConstants.TOP);
			relationshipTo.setVerticalAlignment(SwingConstants.TOP);
			JPanel relationshipLabels = new JPanel(new BorderLayout());
			relationshipLabels.add(relationshipFrom, BorderLayout.WEST);
			relationshipLabels.add(relationshipTo, BorderLayout.EAST);
			relationshipLabels.setBorder(
				BorderFactory.createMatteBorder(3,0,0,0, Color.BLACK));
			JPanel relationshipGrid = new JPanel(new GridLayout(2,1));
			relationshipGrid.add(new JPanel());
			relationshipGrid.add(relationshipLabels);
		relationshipPanel.add(relationshipGrid, BorderLayout.CENTER);
		relationshipPanel.setBorder(BorderFactory.createEmptyBorder(10,0,0,0));

	JPanel coordinationPanel = new JPanel(new BorderLayout());
	coordinationPanel.add(visualizationAPanel, BorderLayout.WEST);
	coordinationPanel.add(visualizationBPanel, BorderLayout.EAST);
	coordinationPanel.add(relationshipPanel, BorderLayout.CENTER);
	coordinationPanel.setBorder(BorderFactory.createEmptyBorder(10,0,0,0));

	// Setup Button Panel
	buttonPanel = new JPanel();
		//buttonPanel.setMaximumSize(new Dimension(500, 10));
		saveButton = new JButton("Save");
		saveButton.setActionCommand("Save");
		saveButton.addActionListener(this);        
		cancelButton = new JButton("Cancel");
		cancelButton.setActionCommand("Cancel");
		cancelButton.addActionListener(this);
	buttonPanel.add(saveButton);
	buttonPanel.add(cancelButton);

	//Add everything to the Frame Panel
	framePanel.add(textPanel);
	framePanel.add(coordinationNamePanel);
	framePanel.add(coordinationPanel);
	framePanel.add(buttonPanel);

	this.setSize(WINDOW_WIDTH, WINDOW_HEIGHT);
	this.getContentPane().add(framePanel);
	fillDefaultValues();
	setPositionCentered(this);
	setResizable(false);
	this.setVisible(true);
}

/** Centers the frame */
private void setPositionCentered(Window window)
{
	int x = (window.getGraphicsConfiguration().getBounds().width / 2) -
		(window.getWidth() / 2);
	int y = (window.getGraphicsConfiguration().getBounds().height / 2) -
		(window.getHeight() / 2);
	window.setLocation(x, y);
}

/**
 * Fills the windows interaction objects (Fields, ComboBoxes, Lists, etc.)
 * with the appropriate values retrieved from the database and sets their
 * initial state.
 */    
private void fillDefaultValues()
{
	Enumeration e;
	UINode node;
	String tableNameFrom, tableNameTo;

	node = link.getStartVisualization().getUINode();
	typeFrom.setText(node.getType());
	tableNameFrom = node.getQuery().getTable();
	tableFrom.setText(tableNameFrom);

	e = node.getTechnologyAdapter().getSupportedActions();
	while(e.hasMoreElements())
	{
		actionFrom.addItem(e.nextElement());
	}

	node = link.getEndVisualization().getUINode();
	typeTo.setText(node.getType());
	tableNameTo = node.getQuery().getTable();
	tableTo.setText(tableNameTo);

	e = node.getTechnologyAdapter().getSupportedActions();
	while(e.hasMoreElements())
	{
		actionTo.addItem(e.nextElement());
	}

	// Set the relationship
	switch (schema.getRelation(tableNameFrom, tableNameTo))
	{
		case DatabaseSchema.ONE_TO_ONE:
			relationshipFrom.setText("1");
			relationshipTo.setText("1");
		break;
		case DatabaseSchema.MANY_TO_ONE:
			relationshipFrom.setText("M");
			relationshipTo.setText("1");
		break;
		case DatabaseSchema.ONE_TO_MANY:
			relationshipFrom.setText("1");
			relationshipTo.setText("M");
		break;
		case DatabaseSchema.NO_RELATION:
			relationshipFrom.setText("");
			relationshipTo.setText("");
		break;
	}


	actionTo.setSelectedIndex(-1);
	String action = coordination.getStartAction();
	if(action.equals(""))
		actionFrom.setSelectedIndex(-1);
	else
		actionFrom.setSelectedItem(action);

	action = coordination.getEndAction();
	if(action.equals(""))
		actionTo.setSelectedIndex(-1);
	else
		actionTo.setSelectedItem(action);
}

/**
 * Ensures that all the fields have some appropriate values in them.  This
 * function does error checking and validation by simply making certain
 * that all fields have something in them.  Further validation could be
 * made in the future.
 *
 * @returns returns true if fields contain valid values, false otherwise
 */    
private boolean validateFields(){
	if(	(actionFrom.getSelectedIndex() != -1) &&
		(actionTo.getSelectedIndex() != -1)
		)
		return true;
	else
		return false;
}

/**
 * Handles action events sent by the various components in the interaction
 * window.  This method simply involves setting the return variables equal
 * to the action types selected and handling action events from the
 * buttons.
 */    
public void actionPerformed(ActionEvent actionEvent)
{
	if(actionEvent.getActionCommand().equals("Save"))
	{
		if(validateFields())
		{
			saveEvent();
		}
	}
	else if(actionEvent.getActionCommand().equals("Cancel"))
		cancelEvent();
}

/** Save changes and update the CoordinationGraph */
private void saveEvent()
{
	// Gets information about the old connection (to replace if needed)
	UINode nodeA = (UINode) link.getStartVisualization().getUINode();
	String actionA = coordination.getStartAction();
	UINode nodeB = (UINode) link.getEndVisualization().getUINode();
	String actionB = coordination.getEndAction();

	// Stores the coordination in the UI's data structure
	Vector v = link.getCoordinations();
	coordination.setStartAction(getFirstAction());
	coordination.setEndAction(getSecondAction());
	if (addingCoordination)
		v.add(coordination);

	try
	{
		// Stores the connection in the CoordinationGraph's data structure
		nodeA.getGraph().getMonitor().acquireWrite();
		// Begin critical section
		// --------------------------------------------------------
			if(!addingCoordination)  // Remove the old connection
			{
				nodeA.removeConnection(actionA,
					new NodeActionPair(nodeB, actionB));
				nodeB.removeConnection(actionB,
					new NodeActionPair(nodeA, actionA));
			}

			// Add the connection
			nodeA.addConnection(getFirstAction(),
				new NodeActionPair(nodeB, getSecondAction()));
			nodeB.addConnection(getSecondAction(),
				new NodeActionPair(nodeA, getFirstAction()));
		// --------------------------------------------------------
		// End critical section
		nodeA.getGraph().getMonitor().releaseWrite();
		this.hide();
	}
	catch(InterruptedException e)
	{
		System.err.println(
			"Unable to obtain lock for the graph: " + e.toString());
	}
}

/** Cancel everything and undo creation of the coordination */
private void cancelEvent()
{
	if (coordination.getStartAction().equals("") ||
		coordination.getEndAction().equals(""))
	{
		link.getCoordinations().remove(coordination);
		if (link.getCoordinations().size() == 0)
		{
			parent.removeLink(link);
		}
	}
	this.hide();
}

}
