package snap.ui.diagram;

// Allows access to the UIGraph and UINode
import snap.graph.*;

import java.awt.*;
import java.util.*;
import javax.swing.*;

/**
  * Visualization is the UI abstraction of a Visualization object.  The
  * Visualization object encapsulates all the information needed to
  * build the UI representation of a visualization within a layout.
  *
  * @author Nathan Conklin - nathan@conklinfamily.net
  * @author Matt Clement - macleme2@vt.edu
  * @author Chris North - north@cs.vt.edu
  */
public class Visualization
{
    
    private UINode          node;				//Node of visualization
    private DiagramPanel    container;          //Containing object
    private Vector			linkList;   //List of coordinations
    private Icon			visualizationIcon;  //Iconic representation
    private Point           location;           //Painting location

/**
 * Class constructor
 * 
 * @param node          the node of the new Visualization
 * @param xy            the Point location where the object will be painted
 * @param Icon			the iconic representation of the object
 * @param thisContainer the containing object, in this case DiagramPanel
 *
 */
public Visualization(UINode node, DiagramPanel container)
{
	this.node			= node;
	visualizationIcon	= node.getIcon();
	location			= container.getNewVisualizationPoint();
	this.container		= container;
	linkList			= new Vector();
}
    
/**
 * Sets the location where the Visualization should paint to the point
 * specified by parameter.
 *
 * @param newPoint      the new location where the object should paint
 */
public void setLocation(Point newPoint)
{
	location.setLocation(newPoint);
}
    
/**
 * Retrieves the UINode of the Visualization object.
 *
 * @return the UINode of the Visualization object
 */
public UINode getUINode()
{
	return node;
}

/**
 * Retrieves the location where the Visualization object is painting
 * 
 * @return returns the location as type Point where the object is painting
 */
public Point getLocation()
{
	return location;
}
    
/**
 * Calculates the center point of the iconic representation of the
 * Visualization and returns that as a Point object for rendering the
 * associated coordinations in the UI.
 *
 * @return returns a new Point object located at the center of the
 *         visualization icon
 */
public Point getCenterPoint()
{
	return new Point((int)(location.x+(0.5*getIconWidth())),
					 (int)(location.y+(0.5*getIconHeight())));
}

/**
 * Retrieves the list of Links associated with this Visualization.
 *
 * @return the list of Links associated with this Visualization
 */
public Vector getLinks()
{
	return linkList;
}
    
/**
 * Retrieves the height in pixels of the iconic representation of the
 * Visualization object.
 *
 * @return returns the height (in pixels) of the iconic representation
 */
private int getIconHeight()
{
	return visualizationIcon.getIconHeight(); 
}

/**
 * Retrieves the width in pixels of the iconic representation of the
 * Visualization object.
 *
 * @return returns the width (in pixels) of the iconic representation
 */    
private int getIconWidth()
{
	 return visualizationIcon.getIconWidth(); 
}
    
/**
 * Determines if the Visualization is connected by examining the list
 * of associated coordinations.  Returns false if the list is empty and true
 * otherwise.
 *
 * @return returns true if the object is connected and false if it is not
 */
public boolean isLinked()
{
	return !linkList.isEmpty();
}    

/**
 * Determines whether a given point passed in as a parameter is within the
 * graphical drawing space of the Visualization.  Returns true if the
 * point is located within the bounds of the iconic representation and
 * false otherwise.  This test does NOT account for points on the edge of
 * the icon.
 *
 * @param testPoint     the point to test for containment
 *
 * @return returns true if the point is within the bounds of the iconic
 *         representation and false otherwise
 */
public boolean containsPoint(Point testPoint)
{
	if( ( testPoint.x > location.x
			&&
		   testPoint.y > location.y ) 
		&& 
		( testPoint.x < (location.x + getIconWidth()) 
			&& 
		  testPoint.y < (location.y + getIconHeight())
		)
	)
		return true;
	else
		return false;        
}
    
/**
 * Draws the iconic represntation of the SnapVisualizaiton object at its
 * location.
 * 
 * @param g             the graphics context to use for painting
 */
public void paintComponent(Graphics g)
{
	//g.drawImage(visualizationIcon, location.x, location.y, container);
	//System.out.println("Painting " + node.getName() + " at " +
	//	location.x + ", " + location.y);
	visualizationIcon.paintIcon(container, g, location.x, location.y);
}

/**
 * Erases the iconic representation of the Visualization object.  This
 * is accomplished by a simple overlay of appropriate background colors.
 *
 * @param g             the graphics context to use for painting
 */
public void erase(Graphics g)
{
	g.setColor(container.getBackground());
	g.drawRect(getLocation().x, getLocation().y, 
			   getIconWidth(), getIconHeight());
	g.fillRect(getLocation().x, getLocation().y,
			   getIconWidth(), getIconHeight());
}
    
/**
 * Adds a Link to the list of associated links for this 
 * Visualization object
 *
 * @param link    the Link object to be added or associated
 *                      with the visualization object
 */
public void addLink(Link link)
{
	linkList.add(link);
}

/**
 * Removes a coordination from the list of associated coordinations for this 
 * Visualization object
 *
 * @param link    the Link object to be removed from the
 *                      Visualization object
 */
public void removeLink(Link link)
{
	linkList.remove(link);
}

}
