package snap.ui.diagram;

import snap.graph.*;
import snap.adapter.*;
import snap.database.*; // for the Query

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.sql.*;

/**
  * VisualizationSettings is the component that obtains visualization
  * information from the user.  This interaction window determines the type
  * of the visualization, it's name, and the table and fields that this
  * visualization wraps.  The entire interaction window is contained within
  * this class and all associated componenets of the window.  This class
  * also involves quite a bit of database interaction to obtain the
  * aforementioned information.  The interaction window has two "modes" an
  * edit mode and an add mode and is thus used for adding and editing
  * visualizations.  Be careful when working with the window that the state,
  * or mode, is properly set.
  *
  * @author Nathan Conklin - nathan@conklinfamily.net
  * @author Matt Clement - macleme2@vt.edu
  * @author Chris North - north@cs.vt.edu
  */
public class VisualizationSettings extends JFrame 
implements ActionListener
{
    
private JPanel framePanel;
private JComboBox  visualizationType;
private JTextField visualizationName;
private JPanel     imagePanel;
private JTextArea  actionTypes;
private JComboBox  tableChooser;
private JScrollPane      listScrollPane;
private JList            fieldList;
private DefaultListModel listModel;

private JButton saveButton;
private JButton editButton;
private JButton cancelButton;

private DatabaseSchema schema;
private boolean   initializationComplete;

private Toolkit  toolkit;
private Icon     icon;
private Graphics g;

private Visualization visualization;

/**
 * Class constructor takes the data connection
 * and the Visualization object to edit as arguments.
 * 
 * @param vis			the Visualization object to begin editing
 * @param c				the connection to the database data
 */     
public VisualizationSettings(Visualization vis, DatabaseSchema schema)
{
	/*initializationComplete is used merely as a safety check should
	something go awry during the intialization of the setupWindow(), etc.*/
	initializationComplete = false;
	
	visualization	= vis;
	this.schema		= schema;
	toolkit			= Toolkit.getDefaultToolkit();
	listModel		= new DefaultListModel();

	framePanel		= new JPanel();
	framePanel.setLayout(new BoxLayout(framePanel, BoxLayout.Y_AXIS));
	this.setTitle("View Properties: " + vis.getUINode().getName());
	this.createRootPane();
	this.frameInit();
	setupWindow();
	
	initializationComplete = true;
}

/**
 * Sets up the interaction window with its default constituant objects and
 * initializes them to their default state.  The window is made up of
 * several panels each containing related objects.
 */
protected void setupWindow()
{
	// Set up the feed-forward text
	JPanel textPanel = new JPanel(new BorderLayout());
		JTextArea textArea = new JTextArea(
			"\nSelect the table and fields to be shown " +
			"in the visualization:\n");
		textArea.setLineWrap(true);
		textArea.setEditable(false);
		textArea.setBackground(textPanel.getBackground());
	textPanel.add(textArea);

	// Set up the Query Panel
	// Set up the Labels
		JLabel tableLabel = new JLabel("Table:");
		JLabel fieldLabel = new JLabel("Fields:");
		JPanel dataLabelGrid = new JPanel(new GridLayout(2, 1));
		dataLabelGrid.add(tableLabel);
		tableLabel.setBorder(BorderFactory.createEmptyBorder(0,0,5,0));
		dataLabelGrid.add(fieldLabel);
		JPanel dataLabels = new JPanel(new BorderLayout());
		dataLabels.add(dataLabelGrid, BorderLayout.NORTH);
		dataLabels.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

	// Set up the data choices
		tableChooser = new JComboBox();
		tableChooser.addActionListener(this);
		fieldList = new JList(listModel);
		listScrollPane = new JScrollPane(fieldList);
		JPanel dataChoices = new JPanel(new BorderLayout());
		dataChoices.add(tableChooser, BorderLayout.NORTH);
		dataChoices.add(listScrollPane, BorderLayout.CENTER);
		dataChoices.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

	JPanel queryPanel = new JPanel(new BorderLayout());
	queryPanel.add(dataLabels, BorderLayout.WEST);
	queryPanel.add(dataChoices, BorderLayout.CENTER);
	queryPanel.setBorder(BorderFactory.createTitledBorder("Data"));

	// Set up the Visualization Panel
	// Visualization Name and Type
		JLabel visualizationNameLabel = new JLabel("Name:");
		JLabel visualizationTypeLabel = new JLabel("Type:");
		
		JPanel nameAndTypeLabels = new JPanel(new GridLayout(2,1));
		nameAndTypeLabels.add(visualizationNameLabel);
		nameAndTypeLabels.add(visualizationTypeLabel);
		nameAndTypeLabels.setBorder(BorderFactory.createEmptyBorder(0,0,0,5));

		visualizationName = new JTextField();
		visualizationName.setText(visualization.getUINode().getName());
		visualizationName.setEditable(false);

		visualizationType = new JComboBox();                
		visualizationType.addActionListener(this);

		JPanel nameAndTypes = new JPanel(new GridLayout(2,1));
		nameAndTypes.add(visualizationName);
		nameAndTypes.add(visualizationType);

		JPanel nameAndTypePanel = new JPanel(new BorderLayout());
		nameAndTypePanel.add(nameAndTypeLabels, BorderLayout.WEST);
		nameAndTypePanel.add(nameAndTypes, BorderLayout.CENTER);
		nameAndTypePanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

	// Visualization Icon and Actions
		JLabel iconLabel = new JLabel("Icon:");

		JPanel iconPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		JLabel icon = new JLabel(visualization.getUINode().getIcon());
		iconPanel.add(icon);

		JPanel iconAndLabel = new JPanel(new BorderLayout());
		iconAndLabel.add(iconLabel, BorderLayout.WEST);
		iconAndLabel.add(iconPanel, BorderLayout.CENTER);

		JLabel actionLabel = new JLabel("Actions:");
		actionLabel.setBorder(BorderFactory.createEmptyBorder(0,0,0,5));
		actionTypes = new JTextArea("\n\n");
		actionTypes.setEditable(false);
		JScrollPane actionScrollPane = new JScrollPane(actionTypes);

		JPanel actionPanel = new JPanel(new BorderLayout());
		actionPanel.add(actionLabel, BorderLayout.WEST);
		actionPanel.add(actionScrollPane, BorderLayout.CENTER);

		JPanel iconAndActionsPanel = new JPanel(new GridLayout(1, 2));
		iconAndActionsPanel.add(iconAndLabel);
		iconAndActionsPanel.add(actionPanel);
		iconAndActionsPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

	JPanel visualizationPanel = new JPanel(new BorderLayout());
	visualizationPanel.add(nameAndTypePanel, BorderLayout.NORTH);
	visualizationPanel.add(iconAndActionsPanel, BorderLayout.CENTER);
	visualizationPanel.setBorder(
		BorderFactory.createTitledBorder("Visualization"));

	// Set up the Button panel
		JPanel buttonPanel = new JPanel();
		saveButton = new JButton("Save");
		saveButton.setSelected(true);
		saveButton.setActionCommand("Save");
		saveButton.addActionListener(this);

		cancelButton = new JButton("Cancel");
		cancelButton.setActionCommand("Cancel");
		cancelButton.addActionListener(this);        
	
	buttonPanel.add(saveButton);
	buttonPanel.add(cancelButton);

	//Add each of these panels to the main panel
	framePanel.add(textPanel);
	framePanel.add(queryPanel);
	framePanel.add(visualizationPanel);
	framePanel.add(buttonPanel);

	setSize(350, 450);
	getContentPane().add(framePanel);
	fillDefaultValues();
	setPositionCentered(this);
	setResizable(false);
	this.setVisible(true);
}

/** Centers the frame */
private void setPositionCentered(Window window)
{
	int x = (window.getGraphicsConfiguration().getBounds().width / 2) -
		(window.getWidth() / 2);
	int y = (window.getGraphicsConfiguration().getBounds().height / 2) -
		(window.getHeight() / 2);
	window.setLocation(x, y);
}

/**
 * Clears the window, its fields, and sets the fields in preparation for
 * use by the user.  
 *
 * @returns returns true if there were no errors and false if an SQL error
 *          occurred during the processing
 */
protected void clearWindow()
{
	visualizationName.setText(visualization.getUINode().getName());
	fillDefaultValues();    
}
    
/**
 * Fills the windows interaction objects (Fields, ComboBoxes, Lists, etc.)
 * with the appropriate values retrieved from the database and sets their
 * initial state.
 */
private void fillDefaultValues()
{
	visualizationType.addItem(visualization.getUINode().getType());
	visualizationType.setSelectedIndex(0);
	visualizationType.setEditable(false);

	fillActions();		//Fill the table of supported actions
	fillTableInfo();    //Fill the information for available tables
	fillFieldInfo();	//Fill the corresponding information for fields
	loadQuery(visualization.getUINode().getQuery());
}

/**
 * Fills the action text area with the actions corresponding to the
 * visualization type selected by the user and attempts to load the icon
 * for that visualization from the database.
 */
private void fillActions()
{
	String actions = "";
	actionTypes.setEditable(true);  //Prevents unauthorized editing

	if(visualizationType.getSelectedIndex() > -1)
	{
		UINode node = visualization.getUINode();
		Enumeration e =
			node.getTechnologyAdapter().getSupportedActions();
		while(e.hasMoreElements())
		{
			actions += (String)e.nextElement() + "\n";
		}
	}
	actionTypes.setText(actions);
	actionTypes.setEditable(false); //Lock text area again
}

/**
 * Fills the Table ComboBox with the tables available from the database and
 * sets the selection as appropriate.  If the window is in EDIT mode, then
 * the SQL query is parsed to determine the appropriate selection.
 */
private void fillTableInfo()
{
	if(tableChooser.getItemCount() <= 0)
	{
		Enumeration e = schema.getTables().elements();
		while (e.hasMoreElements())
			tableChooser.addItem((String)e.nextElement());
	}
}

/**
 * Fills the field list component with the fields available for the table
 * selected in the table chooser ComboBox and defaults to have them all
 * selected.
 */
private void fillFieldInfo()
{
	if(tableChooser.getSelectedIndex() > -1)
	{
		listModel.clear();
		Vector columns = schema.getColumns(
			tableChooser.getSelectedItem().toString());
		Enumeration e = columns.elements();
		while (e.hasMoreElements())
			listModel.addElement((String) e.nextElement());

		fieldList.getSelectionModel().setSelectionInterval(
			0, listModel.size() - 1);
	}
	else
		listModel.clear();
}

/** Creates an array of indices in the list for the given selected values */
private int[] getIndices(JList list, Vector selectedValues)
{
	Vector indices = new Vector();
	
	for(int i = 0; i < list.getModel().getSize(); i++)
	{
		String value = list.getModel().getElementAt(i).toString();
		Enumeration e = selectedValues.elements();
		while(e.hasMoreElements())
		{
			String selectedValue = e.nextElement().toString();
			if(selectedValue.equals(value))
				indices.add(new Integer(i));
		}
	}

	int[] indexArray = new int[indices.size()];
	for(int i = 0; i < indexArray.length; i++)
		indexArray[i] = ((Integer) indices.elementAt(i)).intValue();
	return indexArray;
}

/**
 * Sets the UI components to display values determined by the Query
 */
private void loadQuery(Query query)
{
	if(!query.getTable().equals(""))
	{
		tableChooser.setSelectedItem(query.getTable());
		fillFieldInfo();
		int[] indices = getIndices(fieldList, query.getFields());
		fieldList.setSelectedIndices(indices);
	}
}

/**
 * Builds a Query based on the items selected in the table chooser
 * ComboBox and the fieldList with the following format:
 *      SELECT Field1, Field2, etc. FROM Table
 *
 * @return Query built from the users selections
 */
private Query createQuery()
{
	String tableName = "";
	Vector fields = new Vector();
	String whereClause = "";

	tableName = tableChooser.getSelectedItem().toString();

	Object[] itemsSelected = fieldList.getSelectedValues();
	for(int x=0; x < itemsSelected.length; x++)
		fields.add(itemsSelected[x].toString());

	return new Query(tableName, fields, whereClause);
}
    
/**
 * Ensures that all the fields have some appropriate values in them.  This
 * function does error checking and validation by simply making certain
 * that all fields have something in them.  Further validation could be
 * made in the future.
 *
 * @returns returns true if fields contain valid values, false otherwise
 */
private boolean validateFields(){
	if	(
		visualizationName.getText() != null &&
		!visualizationName.getText().equals("")	&&
		visualizationType.getSelectedIndex() != -1	&&
		tableChooser.getSelectedIndex() != -1 &&
		!fieldList.isSelectionEmpty()
		)
		return true;
	else
		return false;
}
    

/**
 * Handles action events sent by the various components in the interaction
 * window.  This method would involve updating the actions and fields
 * associated with visualizations and tables respectively as well as
 * handling the actions associated with the buttons.
 */
public void actionPerformed(ActionEvent actionEvent)
{
	if((actionEvent.getSource() instanceof JComboBox) &&
	  (initializationComplete))
	{
		if(actionEvent.getSource() == visualizationType)
			fillActions();
		else if(actionEvent.getSource() == tableChooser)
			fillFieldInfo();
	}
	else
	{
		if(actionEvent.getActionCommand().equals("Save"))
		{               
			if(validateFields())
			{
				try
				{
					UINode node = (UINode) visualization.getUINode();
					node.getGraph().getMonitor().acquireWrite();
					// Begin critical section
					// --------------------------------------------------------
						node.setQuery(createQuery());
					// --------------------------------------------------------
					// End critical section
					node.getGraph().getMonitor().releaseWrite();
					this.hide(); //hide the window
				}
				catch(InterruptedException e)
				{
					System.err.println(
						"Unable to obtain lock for the graph: " + e.toString());
				}
			}
		}
		else if(actionEvent.getActionCommand().equals("Cancel"))
			this.hide(); //Cancel everything and do NOT add anything   
	}
}


 
}
